// lib/meta-tags.mjs
import { inject } from "@angular/core";
import { Meta } from "@angular/platform-browser";
import { NavigationEnd, Router } from "@angular/router";
import { filter } from "rxjs/operators";
var ROUTE_META_TAGS_KEY = Symbol("@analogjs/router Route Meta Tags Key");
var CHARSET_KEY = "charset";
var HTTP_EQUIV_KEY = "httpEquiv";
var HTTP_EQUIV_SELECTOR_KEY = "http-equiv";
var NAME_KEY = "name";
var PROPERTY_KEY = "property";
var CONTENT_KEY = "content";
function updateMetaTagsOnRouteChange() {
  const router = inject(Router);
  const metaService = inject(Meta);
  router.events.pipe(filter((event) => event instanceof NavigationEnd)).subscribe(() => {
    const metaTagMap = getMetaTagMap(router.routerState.snapshot.root);
    for (const metaTagSelector in metaTagMap) {
      const metaTag = metaTagMap[metaTagSelector];
      metaService.updateTag(metaTag, metaTagSelector);
    }
  });
}
function getMetaTagMap(route) {
  const metaTagMap = {};
  let currentRoute = route;
  while (currentRoute) {
    const metaTags = currentRoute.data[ROUTE_META_TAGS_KEY] ?? [];
    for (const metaTag of metaTags) {
      metaTagMap[getMetaTagSelector(metaTag)] = metaTag;
    }
    currentRoute = currentRoute.firstChild;
  }
  return metaTagMap;
}
function getMetaTagSelector(metaTag) {
  if (metaTag.name) {
    return `${NAME_KEY}="${metaTag.name}"`;
  }
  if (metaTag.property) {
    return `${PROPERTY_KEY}="${metaTag.property}"`;
  }
  if (metaTag.httpEquiv) {
    return `${HTTP_EQUIV_SELECTOR_KEY}="${metaTag.httpEquiv}"`;
  }
  return CHARSET_KEY;
}

// lib/endpoints.mjs
var ANALOG_META_KEY = Symbol("@analogjs/router Analog Route Metadata Key");
var PAGE_ENDPOINTS = import.meta.glob([
  "/src/app/pages/**/*.server.ts"
]);

// lib/route-config.mjs
import { inject as inject2 } from "@angular/core";
import { HttpClient } from "@angular/common/http";
import { firstValueFrom } from "rxjs";
function toRouteConfig(routeMeta) {
  if (routeMeta && isRedirectRouteMeta(routeMeta)) {
    return routeMeta;
  }
  let { meta, ...routeConfig } = routeMeta ?? {};
  if (Array.isArray(meta)) {
    routeConfig.data = { ...routeConfig.data, [ROUTE_META_TAGS_KEY]: meta };
  } else if (typeof meta === "function") {
    routeConfig.resolve = {
      ...routeConfig.resolve,
      [ROUTE_META_TAGS_KEY]: meta
    };
  }
  if (!routeConfig) {
    routeConfig = {};
  }
  routeConfig.resolve = {
    ...routeConfig.resolve,
    load: async (route) => {
      const routeConfig2 = route.routeConfig;
      if (PAGE_ENDPOINTS[routeConfig2[ANALOG_META_KEY].endpointKey]) {
        const { queryParams, fragment: hash, params, parent } = route;
        const segment = parent?.url.map((segment2) => segment2.path).join("/") || "";
        const url = new URL("", import.meta.env["VITE_ANALOG_PUBLIC_BASE_URL"]);
        url.pathname = `${url.pathname.endsWith("/") ? url.pathname : url.pathname + "/"}api/_analog${routeConfig2[ANALOG_META_KEY].endpoint}`;
        url.search = `${new URLSearchParams(queryParams).toString()}`;
        url.hash = hash ?? "";
        Object.keys(params).forEach((param) => {
          url.pathname = url.pathname.replace(`[${param}]`, params[param]);
        });
        url.pathname = url.pathname.replace("**", segment);
        if (globalThis.$fetch) {
          return globalThis.$fetch(url.pathname);
        }
        const http = inject2(HttpClient);
        return firstValueFrom(http.get(`${url.href}`));
      }
      return {};
    }
  };
  return routeConfig;
}
function isRedirectRouteMeta(routeMeta) {
  return !!routeMeta.redirectTo;
}

// lib/markdown-helpers.mjs
import { inject as inject3 } from "@angular/core";
var isNgZoneEnabled = typeof Zone !== "undefined" && !!Zone.root;
function toMarkdownModule(markdownFileFactory) {
  return async () => {
    const createLoader = () => Promise.all([import("@analogjs/content"), markdownFileFactory()]);
    const [{ parseRawContentFile, MarkdownRouteComponent, ContentRenderer }, markdownFile] = await (isNgZoneEnabled ? (
      // We are not able to use `runOutsideAngular` because we are not inside
      // an injection context to retrieve the `NgZone` instance.
      // The `Zone.root.run` is required when the code is running in the
      // browser since asynchronous tasks being scheduled in the current context
      // are a reason for unnecessary change detection cycles.
      Zone.root.run(createLoader)
    ) : createLoader());
    const { content, attributes } = parseRawContentFile(markdownFile);
    const { title, meta } = attributes;
    return {
      default: MarkdownRouteComponent,
      routeMeta: {
        data: { _analogContent: content },
        title,
        meta,
        resolve: {
          renderedAnalogContent: async () => {
            const contentRenderer = inject3(ContentRenderer);
            return contentRenderer.render(content);
          }
        }
      }
    };
  };
}

// lib/constants.mjs
var ENDPOINT_EXTENSION = ".server.ts";
var APP_DIR = "src/app";

// lib/routes.mjs
var FILES = import.meta.glob([
  "/app/routes/**/*.ts",
  "/src/app/routes/**/*.ts",
  "/src/app/pages/**/*.page.ts",
  "/src/app/pages/**/*.page.analog"
]);
var CONTENT_FILES = import.meta.glob(["/src/app/routes/**/*.md", "/src/app/pages/**/*.md"], { query: "?raw", import: "default" });
function createRoutes(files) {
  const filenames = Object.keys(files);
  if (filenames.length === 0) {
    return [];
  }
  const rawRoutesByLevelMap = filenames.reduce((acc, filename) => {
    const rawPath = toRawPath(filename);
    const rawSegments = rawPath.split("/");
    const level = rawSegments.length - 1;
    const rawSegment = rawSegments[level];
    const ancestorRawSegments = rawSegments.slice(0, level);
    return {
      ...acc,
      [level]: {
        ...acc[level],
        [rawPath]: {
          filename,
          rawSegment,
          ancestorRawSegments,
          segment: toSegment(rawSegment),
          level,
          children: []
        }
      }
    };
  }, {});
  const allLevels = Object.keys(rawRoutesByLevelMap).map(Number);
  const maxLevel = Math.max(...allLevels);
  for (let level = maxLevel; level > 0; level--) {
    const rawRoutesMap = rawRoutesByLevelMap[level];
    const rawPaths = Object.keys(rawRoutesMap);
    for (const rawPath of rawPaths) {
      const rawRoute = rawRoutesMap[rawPath];
      const parentRawPath = rawRoute.ancestorRawSegments.join("/");
      const parentRawSegmentIndex = rawRoute.ancestorRawSegments.length - 1;
      const parentRawSegment = rawRoute.ancestorRawSegments[parentRawSegmentIndex];
      rawRoutesByLevelMap[level - 1] ||= {};
      rawRoutesByLevelMap[level - 1][parentRawPath] ||= {
        filename: null,
        rawSegment: parentRawSegment,
        ancestorRawSegments: rawRoute.ancestorRawSegments.slice(0, parentRawSegmentIndex),
        segment: toSegment(parentRawSegment),
        level: level - 1,
        children: []
      };
      rawRoutesByLevelMap[level - 1][parentRawPath].children.push(rawRoute);
    }
  }
  const rootRawRoutesMap = rawRoutesByLevelMap[0];
  const rawRoutes = Object.keys(rootRawRoutesMap).map((segment) => rootRawRoutesMap[segment]);
  sortRawRoutes(rawRoutes);
  return toRoutes(rawRoutes, files);
}
function toRawPath(filename) {
  return filename.replace(
    // convert to relative path and remove file extension
    /^\/(.*?)\/routes\/|^\/(.*?)\/pages\/|\/app\/routes\/|(\.page\.(js|ts|analog)$)|(\.(ts|md|analog)$)/g,
    ""
  ).replace(/\[\.{3}.+\]/, "**").replace(/\[([^\]]+)\]/g, ":$1");
}
function toSegment(rawSegment) {
  return rawSegment.replace(/index|\(.*?\)/g, "").replace(/\.|\/+/g, "/").replace(/^\/+|\/+$/g, "");
}
function toRoutes(rawRoutes, files) {
  const routes2 = [];
  for (const rawRoute of rawRoutes) {
    const children = rawRoute.children.length > 0 ? toRoutes(rawRoute.children, files) : void 0;
    let module = void 0;
    let analogMeta = void 0;
    if (rawRoute.filename) {
      const isMarkdownFile = rawRoute.filename.endsWith(".md");
      module = isMarkdownFile ? toMarkdownModule(files[rawRoute.filename]) : files[rawRoute.filename];
      const endpointKey = rawRoute.filename.replace(/\.page\.(ts|analog)$/, ENDPOINT_EXTENSION);
      const rawEndpoint = rawRoute.filename.replace(/\.page\.(ts|analog)$/, "").replace(/\[\.{3}.+\]/, "**").split(APP_DIR)[1];
      const endpoint = (rawEndpoint || "").replace(/\./g, "/").replace(/\/\((.*?)\)$/, "/-$1-");
      analogMeta = {
        endpoint,
        endpointKey
      };
    }
    const route = module ? {
      path: rawRoute.segment,
      loadChildren: () => module().then((m) => [
        {
          path: "",
          component: m.default,
          ...toRouteConfig(m.routeMeta),
          children,
          [ANALOG_META_KEY]: analogMeta
        }
      ])
    } : { path: rawRoute.segment, children };
    routes2.push(route);
  }
  return routes2;
}
function sortRawRoutes(rawRoutes) {
  rawRoutes.sort((a, b) => {
    let segmentA = deprioritizeSegment(a.segment);
    let segmentB = deprioritizeSegment(b.segment);
    if (a.children.length > b.children.length) {
      segmentA = `~${segmentA}`;
    } else if (a.children.length < b.children.length) {
      segmentB = `~${segmentB}`;
    }
    return segmentA > segmentB ? 1 : -1;
  });
  for (const rawRoute of rawRoutes) {
    sortRawRoutes(rawRoute.children);
  }
}
function deprioritizeSegment(segment) {
  return segment.replace(":", "~~").replace("**", "~~~~");
}
var routes = createRoutes({ ...FILES, ...CONTENT_FILES });

// lib/define-route.mjs
import { inject as inject4 } from "@angular/core";
import { Router as Router2 } from "@angular/router";
import { ActivatedRoute } from "@angular/router";
var defineRouteMeta = (route) => {
  return route;
};
var injectRouter = () => {
  return inject4(Router2);
};
var injectActivatedRoute = () => {
  return inject4(ActivatedRoute);
};

// lib/provide-file-router.mjs
import { ENVIRONMENT_INITIALIZER, makeEnvironmentProviders } from "@angular/core";
import { provideRouter } from "@angular/router";
function provideFileRouter(...features) {
  return makeEnvironmentProviders([
    // TODO: remove type casting after Angular >=15.1.1 upgrade
    // https://github.com/angular/angular/pull/48720
    provideRouter(routes, ...features).ɵproviders,
    {
      provide: ENVIRONMENT_INITIALIZER,
      multi: true,
      useValue: () => updateMetaTagsOnRouteChange()
    }
  ]);
}

// lib/inject-load.mjs
import { Injector, inject as inject5 } from "@angular/core";
import { ActivatedRoute as ActivatedRoute2 } from "@angular/router";
import { map } from "rxjs";
function injectLoad(options) {
  const injector = options?.injector ?? inject5(Injector);
  const route = injector.get(ActivatedRoute2);
  return route.data.pipe(map((data) => data["load"]));
}
export {
  createRoutes,
  defineRouteMeta,
  injectActivatedRoute,
  injectLoad,
  injectRouter,
  provideFileRouter,
  routes
};
//# sourceMappingURL=analogjs-router.mjs.map
